<?php

namespace Service;

use Util\InstanceProvider;
use Util\Utilities;
use Doctrine\ORM\EntityManager;
use \Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Attachment
{
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var Utilities
     */
    private $environment;
    /**
     * @var \Doctrine\ORM\EntityRepository|\Doctrine\Persistence\ObjectRepository
     */
    private $attachmentRepository;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager  = $entityManager;
        $this->environment    = InstanceProvider::getEnvironment();

        $this->attachmentRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment');

    }

    public function downLoadAttachment($attachmentId)
    {

        /** @var Entities\Attachment $attachment */
        $attachment = $this->attachmentRepository->findOneBy(array('id' => $attachmentId));
        if ($attachment) {
            list($name, $size, $type, $fileName) = $this->getParamsForDownload($attachment);
            $this->setParamsForDownload($type, $size, $name, $fileName);
            exit;
        } else {
            throw new \Exception('ERR_NOT_ATTACHMENT');
        }
    }

    /**
     * @param $type
     * @param $size
     * @param $name
     * @param $fileName
     * @param bool $isAttachment
     */
    public function setParamsForDownload($type, $size, $name, $fileName,$isAttachment = true)
    {
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        if($size){
            header("Content-Length: " . $size);
        }

        if($isAttachment){
            header('Content-Disposition: attachment; filename="' . $name.'"');
        }else{
            header('Content-Disposition: inline;filename="' . $name.'"');
        }
        readfile($fileName);
    }

    public function getParamsForDownload(Entities\Attachment $attachment)
    {
        if ($attachment) {
            $name = $attachment->getLabel();
            $size = $attachment->getSizeFile();
            $type = $attachment->getTypeFile();
            $fileName = \Util\Utilities::getFileServerFilesUrl().$attachment->getFileName();
        }
        return array($name, $size, $type, $fileName);
    }

    public function downLoadAttachmentGEP($fileName)
    {
        if ($fileName) {
            if(!file_exists($fileName)){
                $fileName = mb_convert_encoding($fileName, 'UTF-8', 'ISO-8859-1');
                if(!file_exists($fileName)){
                    throw new \Exception('Aquivo não existe');
                }
            }
            $arrayDirs = explode(DIRECTORY_SEPARATOR,$fileName);
            $this->setParamsForDownload('application/x-zip-compressed',null, array_pop($arrayDirs), $fileName);
            exit;
        } else {
            throw new \Exception('Nome do arquivo nulo');
        }
    }

    public function newCallAttachment($callId, $description, $documents, $createdBy, $callRelationship, $subject = null, $callType = null,$isPublic)
    {
        $this->addAttachmentsProductCall($documents, $callRelationship,  $callType,$description,$isPublic);
    }

    public function addAttachmentsProductCall($documents, $callRelationship, $callType,$description,$isPublic)
    {
        if($callType == \Service\Call::TYPE_ISSUE){
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task  $issue */
            $issue = InstanceProvider::getRepository()->getTask()->find($callRelationship);
            $comentid = null;
            if(!empty($description)){
                $comentid = InstanceProvider::getService()->getComment()->saveCommentOnIssuesByProductCallComment($callRelationship, $description,$isPublic);
            }
            $this->updateDocumentTask($documents,$issue,$comentid);

        }else{
            $this->addAttachmentsRichClient(null,$description,$documents,$callRelationship);
        }
    }

    public static function directoryCheck($directory){
        if ( !is_dir($directory) && ! @mkdir($directory, 0777, true)) {
            throw new \InvalidArgumentException(sprintf(
                'O diretório "%s" não pode ser criado.',
                $directory
            ));
        }

        if ( ! is_writable($directory)) {
            throw new \InvalidArgumentException(sprintf(
                'O diretório "%s" não pode tem permissão de escrita.',
                $directory
            ));
        }
    }

    public function addAttachmentsRichClient($productCallId, $description, $documents, $callRelationship = null)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();
            $subject = array();
            foreach ($documents as $fileData) {
                $subject[] = $fileData['name'];
            }

            if($description){
                $commentDate = InstanceProvider::getService()->getComment()->saveCallComment( $description, $callRelationship , Call::TYPE_RICH_CLIENT,'');
            } else{
                $description = implode(', ',$subject);
                $commentDate = InstanceProvider::getService()->getComment()->saveCallComment( "Arquivos $description", $callRelationship , Call::TYPE_RICH_CLIENT,'Adicionou anexo(s) pelo CP');
            }
            $dateString = $commentDate->format('YmdHis');
            $documents = InstanceProvider::getService()->getINTRANETIntegration()->insertFile($documents, $callRelationship, $dateString);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    public function updateDocumentTask(array $documents, \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $task, $commentId = null)
    {
        $filesUploaded = array();
        foreach ($documents as $fileData) {
            $typeFile = $fileData['type'];
            $labelFile = $fileData['name'];
            $sizeFile = $fileData['size'];
            $pathFile = $fileData['path'];
            if ($commentId) {
                $comentario = InstanceProvider::getRepository()->getComments()->findOneBy(array('id' => $commentId));
            } else {
                $comentario = null;
            }
            $attachment = InstanceProvider::getFactory()->getAttachment()->createAttachmentTask($typeFile, $pathFile, $labelFile, $sizeFile, $task, $comentario);
            $this->entityManager->persist($attachment);
            $filesUploaded[] = $attachment;
        }
        $this->entityManager->flush();
        return $filesUploaded;
    }

    private function verifyMultipleUses($filePath){
        $relatedAttachments = $this->attachmentRepository->findBy(array('fileName' => $filePath));
        return count($relatedAttachments) == 1;
    }

    public function deleteAttachmentById($id)
    {
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $attachment
         */

        $zhFileUploader = InstanceProvider::getFileUploaderServe();

        $attachment = $this->attachmentRepository->find($id);
        try {
            if ($this->verifyMultipleUses($attachment->getFileName())) {
                $zhFileUploader->deleteFile($attachment->getFileName());
            }
        } catch (\Exception $e) {
            // If something goes wrong with deleteFile(), we should do nothing, only remove it from database
        }
        $this->entityManager->remove($attachment);

        $this->entityManager->flush();
    }

    public function deleteProductCallAttachments($idPMAttachment,$isIssue,$dir) {
        /** @var  \Teknisa\Model\Teknisa\Entities\CpCallAttachment $callAttachment */

        $cpCallAttachmentRepository = InstanceProvider::getRepository()->getCallAttachment();
        $INTRANETIntegration = InstanceProvider::getService()->getINTRANETIntegration();
        if ($isIssue) {
            $callAttachment = $cpCallAttachmentRepository->findOneBy(array('callRelationshipAttachment' => $idPMAttachment));
            $this->deleteAttachmentById($idPMAttachment);
        } else {
            $callAttachment = $cpCallAttachmentRepository->findOneBy(array('nrcallattachment' => $idPMAttachment));
            if($callAttachment){
                $INTRANETIntegration->deleteAttachment($callAttachment);
            }else{
                $INTRANETIntegration->deleteFile($dir);
            }
        }
        if($callAttachment){
            $this->entityManager->remove($callAttachment);
            $this->entityManager->flush();
        }
    }

    public function getPageRequisite($nrsoliserv){

        $fileValues  =  InstanceProvider::getService()->getINTRANETIntegration()->getPageRequisite($nrsoliserv);
        if($fileValues){
            $this->setParamsForDownload($fileValues['extension'],null,$fileValues['name'],$fileValues['fileName'],false);
        }else{
            echo 'File not found';
        }
    }

    public function insertFileSpecificationTaskS($nrsoliserv, $document){

        $fileValues  =  InstanceProvider::getService()->getINTRANETIntegration()->insertFileSpecificationTaskS($document, $nrsoliserv);
        if($fileValues){
            echo 'ok';
        }else{
            echo 'File not found';
        }
    }

    public function addDocumentComment($commentId, array $documents){
        /** @var Entities\Comments $comment */
        $comment  = InstanceProvider::getRepository()->getComments()->find($commentId);
        $filesUploaded = $this->updateDocumentComment($documents, $comment);
        $this->entityManager->flush();
        return $filesUploaded;
    }

    public function updateDocumentComment(array $documents, \Zeedhi\DevOps\Model\Projectmanagement\Entities\Comments $comments){
        $filesUploaded = array();
        foreach($documents as $fileData) {
            $typeFile = $fileData['type'];
            $labelFile = $fileData['name'];
            $sizeFile = $fileData['size'];
            $pathFile = $fileData['path'];
            $attachment= InstanceProvider::getFactory()->getAttachment()->createAttachmentComment($typeFile, $pathFile, $labelFile, $sizeFile, $comments);
            $this->entityManager->persist($attachment);
            $filesUploaded[] = $attachment;
        }
        $this->entityManager->flush();
        return $filesUploaded;
    }
}
